// Self extraction packer
// 28.05.2003 Andreas Martin


#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#include <string.h>
#include <stdio.h>
#include <shellapi.h>
#include <stdarg.h>
#include "zlib.h"
#include "packerres.h"

static BOOL CALLBACK DialogFunc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam);

#define PM_SHELL_FINISHED WM_USER+1
#define PACKERNAME "Neues Paket erstellen.EXE"
#define PRGNAME "Packer"
#define VERSION "V1.2"
//#define USE_HIDDEN_KEY

#define CX_DIALOG			327
#define CY_DIALOG			131
#define CY_DIALOG_AND_PWD	245

#define SEEK_STR_END(x) while(*(x)++)
#define NELEMS(x) (sizeof(x)/sizeof((x)[0]))
#define MIN(a,b) ((a) < (b) ? (a) : (b))
#define MAX(a,b) ((a) > (b) ? (a) : (b))

const TCHAR AppKey[]=		"Archiv";
const TCHAR EMPTY[]=		"";

typedef enum {PACKER_MAKENEW, PACKER_RUN} ePackerType;
typedef enum {PACKAGE_INFLATE, PACKAGE_DEFLATE, PACKAGE_COPY} eTransDirection;
typedef enum {TESTFILE_EXIST, TESTFILE_ISDIRECTORY, TESTFILE_ISREADONLY, TESTFILE_GETSIZE, TESTFILE_GETTIME} eTestOptions;

typedef TCHAR FILENAME[_MAX_PATH+1];
typedef TCHAR PASSWORD[21];

typedef struct{
	TCHAR key[sizeof(AppKey)];
	ULONG AppSize;
} sKey;

typedef struct{
	FILENAME packageName, dataFileName;
	FILETIME FileTime;
	ePackerType Type;
	BOOL bGarbled;
	sKey ID;
}sPackage;

typedef struct{
	HANDLE hProcess;
	HANDLE hThread;
}sWaitProcessData, *lpWaitProcessData;

static HANDLE ghwnd;
static HINSTANCE ghinst;
static HBRUSH hBrush;
static FILENAME ModuleName;
static FILENAME Temp_Path, App_Path;
static FILENAME Temp_FileName;
static sPackage PackageInfo;
static TCHAR buffer[1024], *lpBuffer;
static PASSWORD Password;

DWORD Package_GetAppSize(void);
void WaitProcess_Init(HANDLE);
VOID Package_ExtractPassword(VOID);

INT lstrncmp(LPTSTR s1, LPTSTR s2, UINT size, BOOL ignoreCase){

	size=MIN(size,MAX(lstrlen(s1),lstrlen(s2)));

	return CompareString(
		LOCALE_SYSTEM_DEFAULT,
		ignoreCase ? NORM_IGNORECASE : 0,
		s1, size,
		s2, size) -2;
}

LPTSTR lstrstr(LPTSTR s1, LPTSTR s2){
	INT i,j;

	for (i=0; i<lstrlen(s1)-lstrlen(s2); i++){

		if (s1[i]==s2[0]){

			for(j=1; j<lstrlen(s2)+1; j++){
				if (s1[i+j]!=s2[j])
					break;

				if (!s2[j])
					return &s1[i+j];
			}
		}
	}

	return NULL;
}

VOID Garble(LPBYTE buffer, size_t size, LPBOOL Reset){
	static LPTSTR lpPWD=NULL;

	if (*Reset){
		lpPWD=NULL;
		*Reset=FALSE;
	}

	if (!lpPWD)
		lpPWD=Password;

	if (!*lpPWD)
		return;

	while(size--){
		*buffer++^=*lpPWD++;
		if (!*lpPWD)
			lpPWD=Password;
	}
}

LPTSTR LoadRCStringToBuffer(UINT ID){
	LoadString(ghinst, ID, buffer, NELEMS(buffer));
	lpBuffer=buffer;
	SEEK_STR_END(lpBuffer);

	return buffer;
}

INT RCMessage(DWORD Text, UINT Title, UINT Flags, ...){
	va_list va=va_start(va,Flags);
	LPTSTR lpText;

	if (Text < (DWORD)0xFFFF)
		LoadRCStringToBuffer((UINT)Text);
	else{
		lstrcpy(buffer, (LPTSTR)Text);
		lpBuffer=buffer;
		SEEK_STR_END(lpBuffer);
	}

	wvsprintf(lpBuffer, buffer, va);
	lpText=lpBuffer;
	va_end(va);

	SEEK_STR_END(lpBuffer);

	LoadString(ghinst, Title, lpBuffer, NELEMS(buffer)+buffer-lpBuffer);

	return MessageBox(ghwnd, lpText, lpBuffer, Flags);
}

LONGLONG TestFile(LPTSTR name, eTestOptions Options){
	WIN32_FIND_DATA fd;
	HANDLE hFile;

	hFile=FindFirstFile(name,&fd);
	FindClose(hFile);

	switch (Options){
		case TESTFILE_EXIST:
			return hFile!=INVALID_HANDLE_VALUE;

		case TESTFILE_ISDIRECTORY:
			return (fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY);

		case TESTFILE_ISREADONLY:
			return (fd.dwFileAttributes & FILE_ATTRIBUTE_READONLY);

		case TESTFILE_GETSIZE:
			return fd.nFileSizeLow;

		case TESTFILE_GETTIME:
			return *(LONGLONG*)&fd.ftLastWriteTime;
	}
	return 0;
}

ULONG GetAppSize(void){
	static ULONG AppSize=0;
	FILE *f;

	if (!AppSize)
		return AppSize=TestFile(ModuleName, TESTFILE_GETSIZE);

	return AppSize;
}

BOOL DataTrans(FILE *src, FILE *dst, size_t size, eTransDirection TransDirection){
	LONG read;
	z_stream z;
	LPBYTE input_buffer, output_buffer;
	const WORD buffer_size=32*1024;
	const INT Default_Compression=9;
	INT status, count;
	int progress;
	HANDLE hProgress;
	size_t amount=size;
	BOOL Initiated=TRUE;

	switch (TransDirection){
		case PACKAGE_COPY:
			while(size){
				read=fread(buffer,1,min(sizeof(buffer),size),src);
				if (fwrite(buffer,1,read,dst) != read)
					return FALSE;
				size-=read;
			}

			return TRUE;


		case PACKAGE_DEFLATE:
			hProgress=CreateDialog(ghinst,MAKEINTRESOURCE(IDD_PROGRESS),ghwnd,NULL);
			ShowWindow(hProgress,SW_SHOW);
			LoadRCStringToBuffer(IDS_COMPRESSING);
			SetDlgItemText(hProgress,IDT_PROGRESS,buffer);
			SendMessage(GetDlgItem(hProgress,ID_PROGRESSBAR),PBM_SETPOS,0,0l);

			ZeroMemory(&z, sizeof(z));
			input_buffer=malloc(buffer_size);
			output_buffer=malloc(buffer_size);

    		if (Z_OK != (status=deflateInit(&z, Default_Compression)) )
				goto error_deflate;

    		z.avail_in = 0;
			status=Z_OK;

			while (status==Z_OK) {
	    		z.next_out = output_buffer;
	    		z.avail_out = buffer_size;

        		if ( z.avail_in == 0 ) {
            		z.next_in = input_buffer;
            		z.avail_in = fread( input_buffer, 1, buffer_size, src );
					z.avail_in = min(size, z.avail_in);
					size-=z.avail_in;
        		}

				if (!size)
	        		status = deflate( &z, Z_FINISH );
				else
	        		status = deflate( &z, Z_NO_FLUSH );

				if ( (Z_OK != status) && (Z_STREAM_END != status) )
					goto error_inflate;

    			count = buffer_size - z.avail_out;
    			if ( count ){
					Garble(output_buffer, count, &Initiated);
					fwrite( output_buffer, 1, count, dst );
				}

				progress=100*z.total_in/amount;
				SendMessage(GetDlgItem(hProgress,ID_PROGRESSBAR),PBM_SETPOS,progress,0l);
			}

			error_deflate:
				deflateEnd(&z);
				fflush(dst);
				free(input_buffer);
				free(output_buffer);

				LoadRCStringToBuffer(IDS_READY);
				lstrcat(buffer,": %d%%");
				SEEK_STR_END(lpBuffer);
				wsprintf(lpBuffer,buffer,100*(z.total_out+Package_GetAppSize()+sizeof(sPackage))/amount);
				SetDlgItemText(hProgress,IDT_PROGRESS,lpBuffer);
				Sleep(1500);

				DestroyWindow(hProgress);

				if (Z_STREAM_END != status){
					RCMessage(IDS_ERR_COMPRESS,IDS_COMPRESS,MB_ICONERROR);
					return FALSE;
				}

				return TRUE;

		case PACKAGE_INFLATE:
			hProgress=CreateDialog(ghinst,MAKEINTRESOURCE(IDD_PROGRESS),ghwnd,NULL);
			ShowWindow(hProgress,SW_SHOW);
			LoadRCStringToBuffer(IDS_DECOMPRESSING);
			SetDlgItemText(hProgress,IDT_PROGRESS,buffer);
			SendMessage(GetDlgItem(hProgress,ID_PROGRESSBAR),PBM_SETPOS,0,0l);

			ZeroMemory(&z, sizeof(z));
			input_buffer=malloc(buffer_size);
			output_buffer=malloc(buffer_size);

    		if (Z_OK != (status=inflateInit(&z)) )
				goto error_inflate;

    		z.avail_in = 0;
			status=Z_OK;

			while (status!=Z_STREAM_END) {
	    		z.next_out = output_buffer;
	    		z.avail_out = buffer_size;

        		if ( z.avail_in == 0 ) {
            		z.next_in = input_buffer;
            		z.avail_in = fread( input_buffer, 1, buffer_size, src );
					z.avail_in = min(size, z.avail_in);
					size-=z.avail_in;

					Garble(input_buffer, z.avail_in,&Initiated);
        		}

        		status = inflate( &z, Z_NO_FLUSH );

				if ( (Z_OK != status) && (Z_STREAM_END != status) )
					goto error_inflate;

    			count = buffer_size - z.avail_out;
    			if ( count )
					fwrite( output_buffer, 1, count, dst );

				progress=100*z.total_in/amount;
				SendMessage(GetDlgItem(hProgress,ID_PROGRESSBAR),PBM_SETPOS,progress,0l);
			}

			error_inflate:
				inflateEnd(&z);
				fflush(dst);
				free(input_buffer);
				free(output_buffer);

				LoadRCStringToBuffer(IDS_READY);
				SetDlgItemText(hProgress,IDT_PROGRESS,buffer);
				Sleep(500);

				DestroyWindow(hProgress);

				if (Z_STREAM_END != status){
					RCMessage(IDS_ERR_DECOMPRESS,IDS_ERR_DECOMPRESS,MB_ICONERROR);
					return FALSE;
				}

				return TRUE;
	}

	return FALSE;
}

BOOL IsPackage(void){
	sKey ID;
	FILE* fApp;
	static INT isPackage=0;

	if (!isPackage){
		ZeroMemory(&ID, sizeof(ID));
		fApp=fopen(ModuleName,"rb");
		if (fApp==NULL)
			return FALSE;
		fseek(fApp,-sizeof(ID),SEEK_END);
		if(fread(&ID,1,sizeof(ID),fApp)!=sizeof(ID)){
			fclose(fApp);
			return FALSE;
		}
		fclose(fApp);
		isPackage= lstrncmp(ID.key, AppKey, lstrlen(AppKey), FALSE)==0 ? 1 : -1;
	}
	return isPackage==1;
}

BOOL Package_InitInfo(sPackage *lpPackage){
	FILE *fApp;

	ZeroMemory(lpPackage, sizeof(*lpPackage));
	if (IsPackage()){
		fApp=fopen(ModuleName,"rb");
		if (fApp==NULL)
			return FALSE;
		fseek(fApp,-sizeof(*lpPackage),SEEK_END);
		if(fread(lpPackage,1,sizeof(*lpPackage),fApp)!=sizeof(*lpPackage)){
			fclose(fApp);
			return FALSE;
		}
		fclose(fApp);
	}
	else{
		lstrcpy(PackageInfo.ID.key,AppKey);
		PackageInfo.ID.AppSize=GetAppSize();
		PackageInfo.bGarbled=FALSE;
	}

	return TRUE;
}

LPTSTR Package_GetName(void){
	return IsPackage() ? PackageInfo.packageName : EMPTY;
}

LPTSTR Package_SetName(LPTSTR name){
	lstrcpy(PackageInfo.packageName,name);
	return name;
}

LPTSTR Package_GetDataFileName(void){
	return PackageInfo.dataFileName;
}

void Package_SetDataFileName(LPTSTR FileName){
	lstrcpy(PackageInfo.dataFileName, FileName);
}

UINT Package_GetType(void){
	return IsPackage() ? PackageInfo.Type : PACKER_MAKENEW;
}

void Package_SetType(ePackerType Type){
	PackageInfo.Type=Type;
}

DWORD Package_GetAppSize(void){
	return PackageInfo.ID.AppSize;
}

void Package_SetAppSize(DWORD size){
	PackageInfo.ID.AppSize=size;
}

LONGLONG Package_GetFileTime(void){
	return *(LONGLONG*)&PackageInfo.FileTime;
}

void Package_SetFileTime(FILETIME ft){
	PackageInfo.FileTime=ft;
}

BOOL Package_IsGarbled(VOID){
	return PackageInfo.bGarbled;
}

VOID Package_SetGarbled(VOID){
	PackageInfo.bGarbled=TRUE;
}

FILE* Packer_Write(LPTSTR File){
	FILE *fApp, *f;

	fApp=fopen(ModuleName,"rb");
	f=fopen(File,"wb");

	if (f){
		DataTrans(fApp, f, Package_GetAppSize(), PACKAGE_COPY);
		fclose(fApp);
	}

	return f;
}

BOOL Packer_CreateNew(void){
	FILE *f;

	if (lstrstr(ModuleName,PACKERNAME)!=NULL)
		return TRUE;

	if (TestFile(PACKERNAME, TESTFILE_EXIST))
		return TRUE;

	f=Packer_Write(PACKERNAME);
	if (!f){
		RCMessage(IDS_IFO_EXERO, IDS_ERR_FILEACCESS,MB_ICONSTOP);
		return FALSE;
	}
	fclose(f);
	return TRUE;
}

BOOL Package_Write(ePackerType Type){
	FILE *fPackage, *fData;
	FILENAME Name, File, newDataName;
	TCHAR Ext[_MAX_EXT+1];
	LONGLONG ft,lft, fSize;
	SYSTEMTIME st,lst;
	HANDLE hFile;

	lstrcpy(File, Package_GetDataFileName());

	_splitpath(File, NULL,NULL,Name,Ext);
	wsprintf(newDataName,"%s%s",Name,Ext);
	Package_SetDataFileName(newDataName);

	ft=TestFile(File, TESTFILE_GETTIME);
	if (!ft){
		GetLocalTime(&lst);
		SystemTimeToFileTime(&lst,&lft);
		LocalFileTimeToFileTime(&lft,&ft);
	}

	FileTimeToLocalFileTime(&ft,&lft);
	FileTimeToSystemTime(&lft,&lst);

	fSize=TestFile(File, TESTFILE_GETSIZE);

	fData=fopen(File,"rb");
	if (!fData){
		RCMessage(IDS_ERR_DATAOPEN,IDS_ERR_FILEACCESS,MB_ICONSTOP);
		return FALSE;
	}

	wsprintf(File,"%.02d-%.02d-%.02d (%.02d.%.02d) - %s.EXE",lst.wYear,lst.wMonth,lst.wDay,lst.wHour,lst.wMinute,Name);
	wsprintf(Name,"%s%s",App_Path, File);

	if (TestFile(File,TESTFILE_EXIST)){
		if(TestFile(File,TESTFILE_ISREADONLY)){
			RCMessage(IDS_ERR_FILEEXISTRO,IDS_ERR_FILEACCESS,MB_ICONERROR,File);
			fclose(fData);
			return FALSE;
		}
		else{
			if (RCMessage(IDS_IFO_OVERWRITE,IDS_ERR_FILEEXISTRO,MB_YESNO|MB_ICONQUESTION,File)!=IDYES){
				fclose(fData);
				return FALSE;
			}
		}
	}

	fPackage=Packer_Write(File);
	if (!fPackage){
		fclose(fData);
		RCMessage(IDS_ERR_PACKAGEFAIL,IDS_PACKAGECREATE,MB_ICONSTOP);
		return FALSE;
	}

	if (!DataTrans(fData, fPackage,fSize , PACKAGE_DEFLATE)){
		fclose(fData);
		fclose(fPackage);
		RCMessage(IDS_ERR_PACKAGEWRITE,IDS_ERR_FILEACCESS,MB_ICONSTOP);
		return FALSE;
	}

	fclose(fData);

	Package_SetType(Type);
	Package_SetName(File);
	Package_SetFileTime(*(FILETIME*)&ft);

	if (!fwrite(&PackageInfo, 1, sizeof(PackageInfo), fPackage)){
		fclose(fPackage);
		return FALSE;
	}

	fclose(fPackage);
	return TRUE;
}

static BOOL CALLBACK GetPwdDlgFunc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg) {
		case WM_COMMAND:
			switch(LOWORD(wParam)){
				case IDOK:
					GetDlgItemText(hwndDlg, IDE_PWD, Password, NELEMS(Password));
					EndDialog(hwndDlg,1);
					return 1;
			}
			break;
		case WM_CLOSE:
			lstrcpy(Password, EMPTY);
			EndDialog(hwndDlg, 0);
			return 1;
	}

	return FALSE;
}

BOOL Package_Run(void){
	FILE *fPackage, *fData;
	size_t size;
	SYSTEMTIME st;
	LONGLONG ft;
	HANDLE hFile;
	SHELLEXECUTEINFO sInfo;

	ft=Package_GetFileTime();
	fPackage=fopen(ModuleName, "rb");

	if(!fPackage){
		RCMessage(IDS_ERR_PACKAGEOPEN,IDS_ERR_FILEACCESS,MB_ICONSTOP);
		return FALSE;
	}

	size=TestFile(ModuleName, TESTFILE_GETSIZE);
	size-=Package_GetAppSize()+sizeof(PackageInfo);

	wsprintf(Temp_FileName, "%s%s",Temp_Path,Package_GetDataFileName());

	fData=fopen(Temp_FileName,"wb");

	if (!fData){
		RCMessage(IDS_ERR_TEMPCREATE,IDS_ERR_FILEACCESS,MB_ICONSTOP);
		fclose(fPackage);
		return FALSE;
	}

	if (Package_IsGarbled()){
		if (!DialogBox(ghinst,MAKEINTRESOURCE(IDD_GETPWD),ghwnd,GetPwdDlgFunc)){
			fclose(fPackage);
			fclose(fData);

			return FALSE;
		}
	}

	fseek(fPackage,Package_GetAppSize(),SEEK_SET);

	if (!DataTrans(fPackage, fData, size, PACKAGE_INFLATE)){
		fclose(fPackage);
		fclose(fData);
		RCMessage(IDS_ERR_TEMPWRITE,IDS_ERR_FILEACCESS,MB_ICONSTOP);
		return FALSE;
	}

	fclose(fPackage);
	fclose(fData);

	hFile=CreateFile(Temp_FileName,GENERIC_WRITE,FILE_SHARE_WRITE,NULL,OPEN_EXISTING,0,NULL);

	if(hFile!=INVALID_HANDLE_VALUE){
		SetFileTime(hFile,NULL,NULL,&ft);
		FlushFileBuffers(hFile);
		CloseHandle(hFile);
	}
	else{
		RCMessage(IDS_ERR_FILETIMECHANGE,IDS_ERR_FILEACCESS,MB_ICONSTOP);
		return FALSE;
	}

	ZeroMemory(&sInfo,sizeof(sInfo));
	sInfo.cbSize=sizeof(sInfo);
	sInfo.hwnd=ghwnd;
	sInfo.fMask=SEE_MASK_NOCLOSEPROCESS;
	sInfo.lpFile=Temp_FileName;
	sInfo.nShow=SW_NORMAL;

	if (!SUCCEEDED(ShellExecuteEx(&sInfo)))
		return FALSE;

	WaitProcess_Init(sInfo.hProcess);
	return TRUE;
}

void Package_Update(void){
	LONGLONG ft1,ft2;
	ft1=Package_GetFileTime();
	ft2=TestFile(Temp_FileName, TESTFILE_GETTIME);

	if (ft1 && ft2){
		if(ft1<ft2){
			Package_SetDataFileName(Temp_FileName);
			if (Package_Write(PACKER_RUN))
				RCMessage(IDS_IFO_PACKAGECHANGED,IDS_PACKAGEMGR,MB_ICONINFORMATION|MB_TOPMOST);
			else
				RCMessage(IDS_ERR_PACKAGEFAIL,IDS_PACKAGEMGR,MB_ICONSTOP|MB_TOPMOST);

		}
		else
			RCMessage(IDS_IFO_PACKAGENOCHANGE,IDS_PACKAGEMGR,MB_ICONINFORMATION|MB_TOPMOST);
	}

	if (!DeleteFile(Temp_FileName))
		RCMessage(IDS_ERR_TEMPDELETE,IDS_PACKAGEMGR,MB_ICONINFORMATION|MB_TOPMOST);
}

BOOL InitializeApp(void)
{
	TCHAR Drive[_MAX_DRIVE+1], Dir[_MAX_DIR+1];

	GetModuleFileName(NULL, ModuleName, _MAX_PATH);

	_splitpath(ModuleName,Drive,Dir,NULL,NULL);
	wsprintf(App_Path,"%s%s",Drive,Dir);
	if (App_Path[lstrlen(App_Path)-1]!='\\')
		lstrcat(App_Path,"\\");

	if (!Package_InitInfo(&PackageInfo)){
		RCMessage(IDS_ERR_ARCHDAMAGE,IDS_ERR_FILEERROR,MB_ICONSTOP);
		return FALSE;
	}

	GetTempPath(_MAX_PATH, Temp_Path);
	if (Temp_Path[lstrlen(Temp_Path)-1]!='\\')
		lstrcat(Temp_Path,"\\");

	return TRUE;
}

int APIENTRY WinMain(HINSTANCE hinst, HINSTANCE hinstPrev, LPSTR lpCmdLine, int nCmdShow)
{
	WNDCLASS wc;
	HANDLE MainDialog;
	MSG msg;
	LPARAM lParam;
	WPARAM wParam;

	#ifdef USE_HIDDEN_KEY

	SHORT status;

	status=GetAsyncKeyState(VK_SHIFT);
	status&=GetAsyncKeyState(VK_CONTROL);
	if (status>=0)
		return 0;

	#endif

	memset(&wc,0,sizeof(wc));
	wc.lpfnWndProc = DefDlgProc;
	wc.cbWndExtra = DLGWINDOWEXTRA;
	wc.hInstance = hinst;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hIcon = LoadIcon(hinst, MAKEINTRESOURCE(APP_ICON));
	wc.hbrBackground = (HBRUSH) (COLOR_WINDOW + 1);
	wc.lpszClassName = PRGNAME;
	RegisterClass(&wc);

	if (!InitializeApp())
		return 1;

	ghinst=hinst;
	hBrush=CreateSolidBrush(GetSysColor(COLOR_BTNFACE));

	MainDialog=CreateDialogParam(hinst,MAKEINTRESOURCE(IDD_MAINDIALOG),NULL,DialogFunc,Package_GetType());

	while (GetMessage(&msg, (HWND) NULL, 0, 0))
	{
	    if (!IsDialogMessage(MainDialog, &msg))
	    {
	        TranslateMessage(&msg);
	        DispatchMessage(&msg);
	    }
	}

	DestroyWindow(MainDialog);
	DeleteObject(hBrush);

	return 0;
}

DWORD WINAPI WaitForChildProcess(LPVOID Data){
	lpWaitProcessData WPD=Data;

	WaitForInputIdle(WPD->hProcess, INFINITE);
	WaitForSingleObject(WPD->hProcess, INFINITE);

	PostMessage(ghwnd,PM_SHELL_FINISHED,0,(LPARAM)Data);
	return 0;
}

void WaitProcess_Init(HANDLE hProcess){
	lpWaitProcessData WPD;
	DWORD ThreadID;

	WPD=malloc(sizeof(*WPD));
	WPD->hProcess=hProcess;
	WPD->hThread=CreateThread(NULL,0,WaitForChildProcess,WPD,0,&ThreadID);
	if (WPD->hThread==NULL)
		RCMessage(IDS_ERR_FILEWATCH,IDS_PACKAGEMGR,0);
}

void WaitProcess_Destroy(lpWaitProcessData WPD){
	if (WPD->hThread)
		CloseHandle(WPD->hThread);
	if (WPD->hProcess)
		CloseHandle(WPD->hProcess);

	free(WPD);
}

void DisplayFileName(){
	TCHAR DisplayText[_MAX_PATH+1];
	RECT r;
	HDC hDC;
	HWND hwndText;

	Button_Enable(GetDlgItem(ghwnd, IDOK), lstrlen(Package_GetDataFileName()));

	if (!lstrlen(Package_GetDataFileName())){
		LoadRCStringToBuffer(IDS_NOSELECTION);
		SetDlgItemText(ghwnd,IDT_FileName,buffer);
		return;
	}
	lstrcpy(DisplayText, PackageInfo.dataFileName);
	hwndText=GetDlgItem(ghwnd, IDT_FileName);
	GetClientRect(hwndText, &r);
	hDC=GetDC(hwndText);
	DrawText(
		hDC,
		DisplayText,
		-1,
		&r,
		DT_PATH_ELLIPSIS|DT_MODIFYSTRING);
	ReleaseDC(hwndText,hDC);

	SetDlgItemText(ghwnd, IDT_FileName, DisplayText);
}

BOOL CheckPwdInput(void){
	PASSWORD pwd1, pwd2;
	if (!Button_GetState(GetDlgItem(ghwnd, IDCB_PASSWORD)))
		return TRUE;

	GetDlgItemText(ghwnd, IDE_PASSWORD1, pwd1, NELEMS(pwd1));
	GetDlgItemText(ghwnd, IDE_PASSWORD2, pwd2, NELEMS(pwd2));

	if (lstrncmp(pwd1,pwd2,NELEMS(pwd1),FALSE)!=0){
		RCMessage(IDS_ERR_PWDNOTUNIQUE,IDS_ERR_PWD,MB_ICONSTOP);
		return FALSE;
	}

	if (!lstrlen(pwd1)){
		RCMessage(IDS_ERR_PWDNOTENTERED,IDS_ERR_PWD,MB_ICONSTOP);
		return FALSE;
	}

	lstrcpy(Password, pwd1);
	Package_SetGarbled();

	return TRUE;
}

static BOOL CALLBACK DialogFunc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
	UINT uHitTest;

	switch (msg) {
	case WM_INITDIALOG:
		ghwnd=hwndDlg;
		DragAcceptFiles(ghwnd, TRUE);
		Edit_LimitText(GetDlgItem(ghwnd, IDE_PASSWORD1), sizeof(PASSWORD));
		Edit_LimitText(GetDlgItem(ghwnd, IDE_PASSWORD2), sizeof(PASSWORD));
		switch(lParam){
			case PACKER_MAKENEW:	SetWindowText(hwndDlg,PRGNAME " " VERSION);
									SetWindowPos(hwndDlg,HWND_TOPMOST,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE);
									DisplayFileName();
									ShowWindow(ghwnd, SW_SHOW);
									break;

			case PACKER_RUN:		Packer_CreateNew();

									if (!Package_Run())
										PostQuitMessage(0);
									break;
		}
		return TRUE;

	case WM_CTLCOLORSTATIC:
		if ((HANDLE)lParam==GetDlgItem(ghwnd, IDT_FileName)){
			SetTextColor((HDC)wParam, RGB(10,100,10));
			SetBkColor((HDC)wParam,GetSysColor(COLOR_BTNFACE));
			return SetDlgMsgResult(ghwnd, WM_CTLCOLORSTATIC,hBrush);
		}
		return 0;

	case WM_NCHITTEST:
		uHitTest = DefWindowProc(ghwnd, WM_NCHITTEST, wParam, lParam);
		if(uHitTest == HTCLIENT)
			return SetDlgMsgResult(ghwnd, WM_NCHITTEST, HTCAPTION);
		else
			return SetDlgMsgResult(ghwnd, WM_NCHITTEST, uHitTest);

	case PM_SHELL_FINISHED:
		WaitProcess_Destroy((lpWaitProcessData)lParam);
		Package_Update();
		PostQuitMessage(0);
		break;

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
			case IDOK:
				if (CheckPwdInput()){
					Package_Write(PACKER_RUN);
					PostQuitMessage(0);
				}
				return 1;

			case IDCANCEL:
				PostQuitMessage(0);
				return 1;

			case IDCB_PASSWORD:
				SetWindowPos(
					ghwnd,
					NULL,
					0,0,CX_DIALOG,
					Button_GetCheck(GetDlgItem(ghwnd,IDCB_PASSWORD)) ? CY_DIALOG_AND_PWD : CY_DIALOG,
					SWP_NOMOVE | SWP_NOZORDER);
				return 1;

		}
		break;

	case WM_DROPFILES:
		DragQueryFile((HDROP)wParam,0,Temp_FileName,_MAX_PATH);
		if (TestFile(Temp_FileName, TESTFILE_ISDIRECTORY)){
			RCMessage(IDS_ERR_DROPFAIL,IDS_ERROR,MB_ICONINFORMATION);
			return 0;
		}
		Package_SetDataFileName(Temp_FileName);
		DisplayFileName();
		return 0;

	case WM_CLOSE:
		PostQuitMessage(0);
		return TRUE;

	}
	return FALSE;
}
